% Script to create a set of noisy, rotated versions of MNIST digit images
% (C) 2016 Weiran Wang and Miguel A. Carreira-Perpinan

load mnist_all.mat

N = 50; X0 = [train7(1:N,:);test7(1:N,:)];	% images to use
M = 60; deg = linspace(-pi,pi,M+1)';		% 360/M-degree rotations
s = 0.2;					% noise stdev

% Create rotated images
X = zeros(2*N*M,size(X0,2)); angle = zeros(2*M*N,1);
for i=1:2*N
  I = mat2gray(reshape(X0(i,:),28,28)');
  for j=1:M
    I1 = imrotate(I,deg(j)*180/pi,'bilinear','crop')';
    I1 = I1 + randn(size(I1))*s; I1(I1<0)=0; I1(I1>1)=1;
    k = (i-1)*M+j; X(k,:) = I1(:)'; angle(k) = deg(j);
  end
end
X = uint8(255*X);
train_img = X(1:N*M,:); train_angle = angle(1:N*M);
test_img = X(N*M+1:end,:); test_angle = angle(N*M+1:end);

% Create output labels (digit-7 skeletons)
anchor = [-3 3;-2 3;-1 3;0 3;0 2;0 1;0 0;0 -1;0 -2;0 -3;0 -4;-2 0;-1 0;1 0];
A = zeros(M,prod(size(anchor)));
for j=1:M
  tmp = anchor*[cos(deg(j)) sin(deg(j));-sin(deg(j)) cos(deg(j))];
  A(j,:) = tmp(:)';
end
train_skel = repmat(A,N,1); test_skel = train_skel;

save MNISTrotated7.mat train_img train_angle train_skel ...
  test_img test_angle test_skel

exit


% Plot some (image,skeleton) pairs
figure(1);
for i=3:10:N
  for j=1:M
    k = (i-1)*M+j;
    clf;
    subplot(1,2,1);
    imagesc(reshape(train_img(k,:),28,28)'); colormap(gray(256)); axis image;
    title(['image ' num2str(i)]);
    subplot(1,2,2);
    tmp = reshape(train_skel(k,:)',[size(train_skel,2)/2 2]);
    hold on;
    plot(tmp(:,1),tmp(:,2),'ro','MarkerFaceColor','r');
    plot(tmp(1:11,1),tmp(1:11,2),'k-');
    plot(tmp([12 13 7 14],1),tmp([12 13 7 14],2),'k-');
    axis([-5 5 -5 5]); set(gca,'DataAspectRatio',[1 1 1]);
    title(['angle = ' num2str(train_angle(k)*180/pi)]);
    hold off;
    pause;
  end
end

